@@ -1,7 +1,6 @@
 //! Broker trait definition
 use std::sync::Arc;
 use async_trait::async_trait;
-use flume::Receiver;
 use futures::{Future, FutureExt, sink::{Sink}, stream::{Stream, StreamExt}};
 
 use super::{Running, Context};
@@ -26,13 +25,13 @@ pub trait Broker: Sized {
         ctx: &Arc<Context<Self::Item>>,
         item: Self::Item,
         writer: W,
-    ) -> Running<Result<Self::Ok, Self::Error>>
+    ) -> Running<Result<Self::Ok, Self::Error>, Option<Self::Error>>
     where W: Sink<Self::WriterItem, Error = flume::SendError<Self::WriterItem>> + Send + Unpin; // None will stop the loop
 
     /// Handles the result of each op
     /// 
     /// Returns a `None` to stop the whole loop
-    async fn handle_result(res: Result<Self::Ok, Self::Error>) -> Running<()> {
+    async fn handle_result(res: Result<Self::Ok, Self::Error>) -> Running<(), Option<Self::Error>> {
         if let Err(_err) = res {
             #[cfg(feature = "debug")]
             log::error!("{:?}", _err);
@@ -41,19 +40,20 @@ pub trait Broker: Sized {
     }
 
     /// Runs the operation in a loop
-    async fn broker_loop<S, W, H>(
+    async fn broker_loop<S, W, RH, WH>(
         mut self, 
         mut items: S, 
         mut writer: W, 
         ctx: Arc<Context<Self::Item>>,
-        stop: Receiver<()>,
-        reader_handle: H, 
-        writer_handle: H
-    )
+        stop: flume::Receiver<()>,
+        reader_handle: RH, 
+        writer_handle: WH
+    ) -> Result<(), Self::Error>
     where 
         S: Stream<Item = Self::Item> + Send + Unpin,
         W: Sink<Self::WriterItem, Error = flume::SendError<Self::WriterItem>> + Send + Unpin,
-        H: Future + Send,
+        RH: Future + Send,
+        WH: Future + Send,
     {
         let this = &mut self;
         let f = Self::handle_result;
@@ -68,11 +68,21 @@ pub trait Broker: Sized {
                             Running::Continue(res) => {
                                 match f(res).await {
                                     Running::Continue(_) => { },
-                                    Running::Stop => break
+                                    Running::Stop(e) => {
+                                        match e {
+                                            None => return Ok(()),
+                                            Some(err) => return Err(err),
+                                        }
+                                    }
                                 }
                             },
                             // None is used to indicate stopping the loop
-                            Running::Stop => break
+                            Running::Stop(e) => {
+                                match e {
+                                    None => return Ok(()),
+                                    Some(err) => return Err(err)
+                                }
+                            }
                         }
                     }
                 }
@@ -96,5 +106,7 @@ pub trait Broker: Sized {
 
         #[cfg(feature = "debug")]
         log::debug!("Exiting broker loop");
+        
+        Ok(())
     }
 }
\ No newline at end of file
